﻿; CUBASE PRO 11 BATCH EXPORT (SHIFT + CTRL + ALT + E)
;
; Author:  David Delbridge
; Website:  daviddelbridge.com
; Published:  7/31/21
;
; This AutoHotKey script facilitates batch processing of Cubase Pro 11 tracks,
; when the target tracks are arranged in consecutive order and routed to a Group
; Track (e.g., "Master") that contains the insert effects (e.g., equalization,
; compression) and automation (e.g., volume fades) to be applied equally to each
; track.
;
; For mix engineers, this permits hands-free rendering of Group Tracks (busses)
; to audio stems (e.g., Drums, Bass, Vocals, Keys) for distribution to mastering
; engineers and remixers.  For mastering engineers, this permits hands-free bulk
; processing of master and alternate mix versions (e.g., TV/karaoke mix, instru-
; mental mix) and stems en masse.  Simply import all of the mixes to contiguous
; tracks, mute all but the master mix, and apply any desired processing to a
; downstream "master" Group Track or output.  Then, mute the master mix track
; and run this script.  All of your mixes will be processed identically.
;
; To use this script, the Export Audio Mixdown window must be open (but not
; necessarily visible) with all desired parameters specified.  The following
; parameter values are important:
;
; - Channel Selection:  (e.g., Stereo Out)
; - Name:  (e.g., Auto_Cubase_Export)
; - Path:  (e.g., C:\Temp\Cubase\Batch)
; - Conflicts:  Create Unique Filename
; - Keep Dialog Open:  Yes
;
; The following preparations are also required:
;
; - In the Project Window, mute all tracks
; - In the Project Window, unmute any desired effects tracks
; - In the Project Window, unmute any required "master" Group Tracks
; - In the Project Window, select the first track to be processed
;
; To launch the script, press Shift + Ctrl + Alt + E
; 
; Multitasking is (mostly) safe.  When each export completes, you will receive a
; standard Windows alert tone three seconds before AutoHotKey must briefly
; commandeer your mouse and keyboard.  Then, it will return you to your applica-
; tion.  Rendered files are stored in your chosen location with auto-increment-
; ing numbers appended to each successive file.

#NoEnv  						; Recommended for performance and compatibility with future AutoHotkey releases.
#Warn  							; Enable warnings to assist with detecting common errors.
SendMode Input  					; Recommended for new scripts due to its superior speed and reliability.
SetWorkingDir %A_ScriptDir%  				; Ensures a consistent starting directory.

Gui, Font, s10, Verdana  				; Set pop-up window's typeface and size
Gui, Add, Text, w500, % "This AutoHotKey script facilitates batch processing of Cubase Pro 11 tracks, when the target tracks are arranged in consecutive order and routed to a Group Track (e.g., ""Master"") that contains the insert effects (e.g., equalization, compression) and automation (e.g., volume fades) to be applied equally to each track.`n`nFor mix engineers, this permits hands-free rendering of Group Tracks (busses) to audio stems (e.g., Drums, Bass, Vocals, Keys) for distribution to mastering engineers and remixers.  For mastering engineers, this permits hands-free bulk processing of master and alternate mix versions (e.g., TV/karaoke mix, instrumental mix) and stems en masse.  Simply import all of the mixes to contiguous tracks, mute all but the master mix, and apply any desired processing to a downstream ""master"" Group Track or output.  Then, mute the master mix track and run this script.  All of your mixes will be processed identically.`n`nTo use this script, the Export Audio Mixdown window must be open (but not necessarily visible) with all desired parameters specified.  The following parameter values are important:`n`n - Channel Selection:  (e.g., Stereo Out)`n - Name:  (e.g., Auto_Cubase_Export)`n - Path:  (e.g., C:\Temp\Cubase\Batch)`n - Conflicts:  Create Unique Filename`n - Keep Dialog Open:  Yes`n`nThe following preparations are also required:`n`n - In the Project Window, mute all tracks`n - In the Project Window, unmute any desired effects tracks`n - In the Project Window, unmute any required ""master"" Group Tracks`n - In the Project Window, select the first track to be processed`n`nTo launch the script, press Shift + Ctrl + Alt + E.`n`nMultitasking is (mostly) safe.  When each export completes, you will receive a standard Windows alert tone three seconds before AutoHotKey must briefly commandeer your mouse and keyboard.  Then, it will return you to your application.  Rendered files are stored in your chosen location with auto-incrementing numbers appended to each successive file."

Gui, Add, Text, h50, Stem track count:  		; Launch a pop-up window with a form and instructions
Gui, Add, DropDownList, vStemCount x+10, 1||2|3|4|5|6|7|8|9|10|11|12|13|14|15|16|17|18|19|20|21|22|23|24
Gui, Add, Button, x220, Cancel				; The label ButtonCancel (if it exists) will be run when the button is pressed.
Gui, Add, Button, default hp+0 x+10, OK  		; The label ButtonOK (if it exists) will be run when the button is pressed.

+^!e::							; Execute this code block when Shift + Ctrl + Alt + E is pressed

WinGetActiveTitle, Win_Active_Title			; Retrieve the user's currently active window so we can restore it

Gui, Show, , Export Stems				; Display the pop-up window
Return

ButtonOK:						; Execute this code block when the "OK" button is selected
Gui, Submit  						; Save the input from the user to each control's associated variable

SetTitleMatchMode, 1					; Match the start of the window title
WinActivate, Cubase Pro Project				; Activate the "Cubase Pro Project" window
WinActivate, Export Audio Mixdown			; Activate the "Export Audio Mixdown" window (might not be necessary)
WinActivate, Cubase Pro Project				; Not sure why, but we must do this twice
WinActivate, Export Audio Mixdown			; Not sure why, but we must do this twice
WinMove, Export Audio Mixdown, , 0, 0			; Make sure the Export Audio Mixdown window is (still) positioned in the topleft corner

Sleep 500						; Sleep for 0.5 second
Send, m							; Unmute the selected track

Loop %StemCount% {
	Sleep 500					; Sleep for 0.5 second
	Click, 790 830					; Press the "Export Audio" button
	Sleep 500					; Wait 0.5 second, just in case the computer needs time to respond

	WinActivate, %Win_Active_Title%			; Reactivate the user's previously active window

	SetTitleMatchMode, 3				; Match the window title exactly
	WinWait, Perform Audio Export, , 60		; Wait for "Perform Audio Export" progress window to appear
	WinWaitClose, Perform Audio Export, , 60	; Wait for "Perform Audio Export" progress window to disappear

	SoundPlay, *48					; Warn user (Windows Exclamation tone) to stop working while script hijacks mouse
	Sleep 3000					; Give user 3 seconds to comply

	WinGetActiveTitle, Win_Active_Title		; Retrieve the user's currently active window so we can restore it

	SetTitleMatchMode, 1				; Match the start of the window title
	WinActivate, Cubase Pro Project			; Activate the "Cubase Pro Project" window
	WinActivate, Export Audio Mixdown		; Activate the "Export Audio Mixdown" window (might not be necessary)
	WinMove, Export Audio Mixdown, , 0, 0		; Make sure the Export Audio Mixdown window is (still) positioned in the topleft corner

	Sleep 500					; Sleep for 0.5 second
	Send, m						; Re-mute the selected track
	If (A_Index <> StemCount) {			; If we're not finished yet, move to the next track
		Sleep 500				; Sleep for 0.5 second
		Send, {Down}				; Select next track
		Sleep 500				; Sleep for 0.5 second
		Send, m					; Unmute the selected track
		}
	}

MsgBox Cubase Pro 11 has finished exporting your audio stems.	; Display "finished" alert pop-up
Exit							;

GuiEscape:						; Execute this code block when the 'Esc' key is pressed
ButtonCancel:						; Execute this code block when the 'Cancel' button is selected
GuiClose:						; Execute this code block when the window is closed
Gui, Hide						; Hide the pop-up window
Exit							; Exit the application
